#include "abstractvideolistmodel.h"
#ifndef QML_USER_INTERFACE
#include "thumbnailcache.h"
#endif

AbstractVideoListModel::AbstractVideoListModel(QObject *parent) :
    QAbstractListModel(parent)
  #ifndef QML_USER_INTERFACE
    ,m_cache(new ThumbnailCache),
    m_thumbnailPressedRow(-1)
  #endif
{
    m_roleNames[IdRole] = "id";
    m_roleNames[VideoIdRole] = "videoId";
    m_roleNames[FavouriteIdRole] = "favouriteId";
    m_roleNames[PlaylistVideoIdRole] = "playlistVideoId";
    m_roleNames[TitleRole] = "title";
    m_roleNames[ArtistRole] = "artist";
    m_roleNames[ArtistAvatarUrlRole] = "artistAvatarUrl";
    m_roleNames[DateRole] = "date";
    m_roleNames[DurationRole] = "duration";
    m_roleNames[ViewCountRole] = "viewCount";
    m_roleNames[LikesRole] = "likes";
    m_roleNames[DislikesRole] = "dislikes";
    m_roleNames[RatingCountRole] = "ratingCount";
    m_roleNames[RatingRole] = "rating";
#ifndef QML_USER_INTERFACE
    m_roleNames[ThumbnailRole] = "thumbnail";
    m_roleNames[ThumbnailPressedRole] = "thumbnailPressed";
#endif
    m_roleNames[ThumbnailUrlRole] = "thumbnailUrl";
    m_roleNames[LargeThumbnailUrlRole] = "largeThumbnailUrl";
    m_roleNames[UrlRole] = "url";
    m_roleNames[ArchiveRole] = "archive";
    m_roleNames[ServiceRole] = "service";
    m_roleNames[FavouriteRole] = "favourite";
    m_roleNames[SelectedRole] = "selected";
#if QT_VERSION < 0x050000
    this->setRoleNames(m_roleNames);
#endif

#ifndef QML_USER_INTERFACE
    this->connect(m_cache, SIGNAL(thumbnailReady()), this, SLOT(onThumbnailReady()));
#endif
}

AbstractVideoListModel::~AbstractVideoListModel() {
    m_list.clear();
#ifndef QML_USER_INTERFACE
    delete m_cache;
    m_cache = 0;
#endif
}

#if QT_VERSION >= 0x050000
QHash<int, QByteArray> AbstractVideoListModel::roleNames() const {
    return m_roleNames;
}
#endif

void AbstractVideoListModel::clear() {
    this->beginResetModel();
    m_list.clear();
    this->endResetModel();
}

int AbstractVideoListModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent)

    return m_list.size();
}

QVariant AbstractVideoListModel::data(const QModelIndex &index, int role) const {
    switch (role) {
    case IdRole:
        return m_list.at(index.row()).data()->id();
    case VideoIdRole:
        return m_list.at(index.row()).data()->videoId();
    case FavouriteIdRole:
        return m_list.at(index.row()).data()->favouriteId();
    case PlaylistVideoIdRole:
        return m_list.at(index.row()).data()->playlistVideoId();
    case TitleRole:
        return m_list.at(index.row()).data()->title();
    case ArtistRole:
        return m_list.at(index.row()).data()->artist();
    case ArtistAvatarUrlRole:
        return m_list.at(index.row()).data()->artistAvatarUrl();
    case DateRole:
        return m_list.at(index.row()).data()->date();
    case DurationRole:
        return m_list.at(index.row()).data()->duration();
    case ViewCountRole:
        return m_list.at(index.row()).data()->viewCount();
    case LikesRole:
        return m_list.at(index.row()).data()->likes();
    case DislikesRole:
        return m_list.at(index.row()).data()->dislikes();
    case RatingCountRole:
        return m_list.at(index.row()).data()->ratingCount();
    case RatingRole:
        return m_list.at(index.row()).data()->rating();
#ifndef QML_USER_INTERFACE
    case ThumbnailRole:
        return m_cache->thumbnail(m_list.at(index.row()).data()->thumbnailUrl(), QSize(160, 120));
    case ThumbnailPressedRole:
        return QVariant(m_thumbnailPressedRow == index.row());
#endif
    case ThumbnailUrlRole:
        return m_list.at(index.row()).data()->thumbnailUrl();
    case LargeThumbnailUrlRole:
        return m_list.at(index.row()).data()->largeThumbnailUrl();
    case UrlRole:
        return m_list.at(index.row()).data()->url();
    case ServiceRole:
        return m_list.at(index.row()).data()->service();
    case FavouriteRole:
        return m_list.at(index.row()).data()->favourite();
    case SelectedRole:
        return QVariant(m_selectedRows.contains(index.row()));
    default:
        return QVariant();
    }
}

QVariant AbstractVideoListModel::data(int row, const QByteArray &role) const {
    return this->data(this->index(row), this->roleNames().key(role));
}

#ifndef QML_USER_INTERFACE
bool AbstractVideoListModel::setData(const QModelIndex &index, const QVariant &value, int role) {
    switch (role) {
    case ThumbnailPressedRole:
        m_thumbnailPressedRow = value.toInt();
        break;
    default:
        return false;
    }

    emit dataChanged(index, index);
    return true;
}
#endif

QSharedPointer<VideoItem> AbstractVideoListModel::get(int row) const {
    if ((row >= 0) && (row < m_list.size())) {
        return QSharedPointer<VideoItem>(m_list.at(row));
    }

    return QSharedPointer<VideoItem>();
}

#ifdef QML_USER_INTERFACE
VideoItem* AbstractVideoListModel::getFromQML(int row) const {
    if ((row >= 0) && (row < m_list.size())) {
        return m_list.at(row).data();
    }

    return 0;
}
#endif

void AbstractVideoListModel::insertVideo(int row, QSharedPointer<VideoItem> video) {
    this->beginInsertRows(QModelIndex(), row, row);
    m_list.insert(row, video);
    this->endInsertRows();
}

void AbstractVideoListModel::appendVideo(QSharedPointer<VideoItem> video) {
    this->beginInsertRows(QModelIndex(), this->rowCount(), this->rowCount());
    m_list.append(video);
    this->endInsertRows();
}

bool AbstractVideoListModel::removeVideo(const QVariant &value, int role) {
    QModelIndexList indexes = this->match(this->index(0), role, value, 1, Qt::MatchExactly);

    if (indexes.isEmpty()) {
        return false;
    }

    return this->removeRow(indexes.first().row());
}

bool AbstractVideoListModel::removeRow(int row, const QModelIndex &parent)
{
    Q_UNUSED(parent)

    if ((row >= 0) && (row < m_list.size())) {
        this->beginRemoveRows(QModelIndex(), row, row);
        m_list.takeAt(row).clear();
        this->endRemoveRows();

        return true;
    }

    return false;
}

void AbstractVideoListModel::selectRow(int row) {
    if (!m_selectedRows.contains(row)) {
        m_selectedRows.append(row);
        emit dataChanged(this->index(row), this->index(row));
    }
}

void AbstractVideoListModel::selectRows(const QList<int> &rows) {
    foreach (int row, rows) {
        if (!m_selectedRows.contains(row)) {
            m_selectedRows.append(row);
        }
    }

    emit dataChanged(this->index(0), this->index(this->rowCount() - 1));
}

void AbstractVideoListModel::deselectRow(int row) {
    if (m_selectedRows.contains(row)) {
        m_selectedRows.removeOne(row);
        emit dataChanged(this->index(row), this->index(row));
    }
}

void AbstractVideoListModel::deselectRows(const QList<int> &rows) {
    foreach (int row, rows) {
        m_selectedRows.removeOne(row);
    }

    emit dataChanged(this->index(0), this->index(this->rowCount() - 1));
}

void AbstractVideoListModel::toggleSelected(int row) {
    if (!m_selectedRows.contains(row)) {
        m_selectedRows.append(row);
    }
    else {
        m_selectedRows.removeOne(row);
    }

    emit dataChanged(this->index(row), this->index(row));
}

void AbstractVideoListModel::selectAll() {
    m_selectedRows.clear();

    for (int i = 0; i < this->rowCount(); i++) {
        m_selectedRows.append(i);
    }

    emit dataChanged(this->index(0), this->index(this->rowCount() - 1));
}

void AbstractVideoListModel::selectNone() {
    m_selectedRows.clear();

    if (this->rowCount() > 0) {
        emit dataChanged(this->index(0), this->index(this->rowCount() - 1));
    }
}

QList< QSharedPointer<VideoItem> > AbstractVideoListModel::selectedItems() const {
    QList< QSharedPointer<VideoItem> > items;

    for (int i = 0; i < m_selectedRows.size(); i++) {
        items.append(this->get(m_selectedRows.at(i)));
    }

    return items;
}

#ifdef QML_USER_INTERFACE
QList<VideoItem*> AbstractVideoListModel::selectedItemsFromQML() const {
    QList<VideoItem*> items;

    for (int i = 0; i < m_selectedRows.size(); i++) {
        items.append(this->getFromQML(m_selectedRows.at(i)));
    }

    return items;
}
#endif

#ifndef QML_USER_INTERFACE
void AbstractVideoListModel::onThumbnailReady() {
    emit dataChanged(this->index(0), this->index(this->rowCount() - 1));
}
#endif
