#ifndef PLAYLISTITEM_H
#define PLAYLISTITEM_H

#include "enums.h"
#include <QObject>
#include <QUrl>
#include <QDomElement>
#include <QVariantMap>

class PlaylistItem : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString id
               READ id
               CONSTANT)
    Q_PROPERTY(QString playlistId
               READ playlistId
               CONSTANT)
    Q_PROPERTY(QString title
               READ title
               NOTIFY titleChanged)
    Q_PROPERTY(QString artist
               READ artist
               NOTIFY artistChanged)
    Q_PROPERTY(QString artistId
               READ artistId
               NOTIFY artistIdChanged)
    Q_PROPERTY(QUrl artistAvatarUrl
               READ artistAvatarUrl
               NOTIFY artistAvatarUrlChanged)
    Q_PROPERTY(QString description
               READ description
               NOTIFY descriptionChanged)
    Q_PROPERTY(int duration
               READ duration
               NOTIFY durationChanged)
    Q_PROPERTY(QUrl thumbnailUrl
               READ thumbnailUrl
               NOTIFY thumbnailUrlChanged)
    Q_PROPERTY(QUrl largeThumbnailUrl
               READ largeThumbnailUrl
               NOTIFY largeThumbnailUrlChanged)
    Q_PROPERTY(QString date
               READ date
               NOTIFY dateChanged)
    Q_PROPERTY(int videoCount
               READ videoCount
               NOTIFY videoCountChanged)
    Q_PROPERTY(bool isPrivate
               READ isPrivate
               NOTIFY isPrivateChanged)
    Q_PROPERTY(Services::VideoService service
               READ service
               NOTIFY serviceChanged)

public:
    explicit PlaylistItem(QObject *parent = 0);
    explicit PlaylistItem(PlaylistItem *playlist, QObject *parent = 0);
    ~PlaylistItem() {}
    Q_INVOKABLE void loadPlaylist(PlaylistItem *playlist);
    Q_INVOKABLE void loadAlbum(PlaylistItem *playlist);
    void loadDailymotionPlaylist(const QVariantMap &playlist);
    void loadYouTubePlaylist(const QDomNode &playlist);
    void loadVimeoAlbum(const QDomElement &album);
    inline QString id() const { return m_id; }
    inline void setId(const QString &id) { m_id = id; }
    inline QString playlistId() const { return m_playlistId; }
    inline void setPlaylistId(const QString &id) { m_playlistId = id; }
    inline QString title() const { return m_title; }
    inline void setTitle(const QString &title) { m_title = title; emit titleChanged(title); }
    inline QString artist() const { return m_artist; }
    inline void setArtist(const QString &artist) { m_artist = artist; emit artistChanged(artist); }
    inline QString artistId() const { return m_artistId; }
    inline void setArtistId(const QString &id) { m_artistId = id; emit artistIdChanged(id); }
    inline QUrl artistAvatarUrl() const { return m_artistAvatarUrl; }
    inline void setArtistAvatarUrl(const QUrl &url) { m_artistAvatarUrl = url; emit artistAvatarUrlChanged(url); }
    inline QString description() const { return m_description; }
    inline void setDescription(const QString &description) { m_description = description; emit descriptionChanged(description); }
    inline int duration() const { return m_duration; }
    inline void setDuration(int duration) { m_duration = duration; emit durationChanged(duration); }
    inline QUrl thumbnailUrl() const { return m_thumbnailUrl; }
    inline void setThumbnailUrl(const QUrl &url) { m_thumbnailUrl = url; emit thumbnailUrlChanged(url); }
    inline QUrl largeThumbnailUrl() const { return m_largeThumbnailUrl; }
    inline void setLargeThumbnailUrl(const QUrl &url) { m_largeThumbnailUrl = url; emit largeThumbnailUrlChanged(url); }
    inline QString date() const { return m_date; }
    inline void setDate(const QString &date) { m_date = date; emit dateChanged(date); }
    inline int videoCount() const { return m_videoCount; }
    inline void setVideoCount(int count) { m_videoCount = count; emit videoCountChanged(count); }
    inline bool isPrivate() const { return m_private; }
    inline void setPrivate(bool isPrivate) { m_private = isPrivate; emit isPrivateChanged(isPrivate); }
    inline Services::VideoService service() const { return m_service; }
    inline void setService(Services::VideoService service) { m_service = service; emit serviceChanged(service); }
    inline bool operator ==(PlaylistItem *other) { return other->playlistId() == this->playlistId(); }

signals:
    void titleChanged(const QString &title);
    void artistChanged(const QString &artist);
    void artistIdChanged(const QString &id);
    void artistAvatarUrlChanged(const QUrl &url);
    void descriptionChanged(const QString &description);
    void durationChanged(int duration);
    void thumbnailUrlChanged(const QUrl &url);
    void largeThumbnailUrlChanged(const QUrl &url);
    void dateChanged(const QString &date);
    void videoCountChanged(int count);
    void isPrivateChanged(bool isPrivate);
    void serviceChanged(Services::VideoService service);

private:
    QString m_id;
    QString m_playlistId;
    QString m_title;
    QString m_artist;
    QString m_artistId;
    QUrl m_artistAvatarUrl;
    QString m_description;
    int m_duration;
    QString m_date;
    QUrl m_thumbnailUrl;
    QUrl m_largeThumbnailUrl;
    int m_videoCount;
    bool m_private;
    Services::VideoService m_service;
};

#endif // PLAYLISTITEM_H
