#ifndef VIMEOACCOUNTSLISTMODEL_H
#define VIMEOACCOUNTSLISTMODEL_H

#include <QSqlTableModel>

class VimeoAccountsListModel : public QSqlTableModel
{
    Q_OBJECT

    Q_PROPERTY(int count
               READ rowCount
               NOTIFY countChanged)

public:
    enum Roles {
        DisplayNameRole = Qt::DisplayRole,
        UsernameRole,
        AccessTokenRole,
        TokenSecretRole,
        ActiveRole
    };

public:
    explicit VimeoAccountsListModel(QObject *parent = 0);
    ~VimeoAccountsListModel();
#if QT_VERSION >= 0x050000
    QHash<int, QByteArray> roleNames() const;
#endif
    QVariant data(const QModelIndex &idx, int role) const;
    Q_INVOKABLE QVariant data(int row, const QByteArray &role) const;

public slots:
    void switchAccount(int row);
    void addAccount(const QString &displayName, const QString &token, const QString &secret);
    void deleteAccount(int row);

private:
    void setActiveAccount(int row);

signals:
    void accountSelected(const QString &user, const QString &token, const QString &secret);
    void accountAdded(const QString &user, const QString &token, const QString &secret);
    void accountDeleted();
    void info(const QString &message);
    void error(const QString &errorString);
    void countChanged(int count);

private:
    QHash<int, QByteArray> m_roleNames;
};

#endif // VIMEOACCOUNTSLISTMODEL_H
