#include "youtubeplaylistlistmodel.h"
#include "feedurls.h"
#include "youtube.h"
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QTimer>
#if QT_VERSION >= 0x050000
#include <QUrlQuery>
#endif

YouTubePlaylistListModel::YouTubePlaylistListModel(QObject *parent) :
    AbstractPlaylistListModel(parent),
    m_loading(true),
    m_moreResults(true),
    m_error(false),
    m_offset(1)
{
}

void YouTubePlaylistListModel::clear() {
    AbstractPlaylistListModel::clear();
    this->setLoading(false);
    this->setOffset(1);
    this->setMoreResults(true);
    this->setError(false);
}

void YouTubePlaylistListModel::reset() {
    if (!this->loading()) {
        this->clear();
        this->getMorePlaylists();
    }
}

void YouTubePlaylistListModel::setFeed(const QUrl &feed) {
    if (feed != this->feed()) {
        m_feed = feed;
    }
}

void YouTubePlaylistListModel::getPlaylists(QUrl feed) {
    if (!feed.isEmpty()) {
        this->setFeed(feed);
    }
    else {
        feed = this->feed();
    }
#if QT_VERSION >= 0x050000
    QUrlQuery query(feed);
    query.addQueryItem("fields", YOUTUBE_PLAYLIST_FIELDS);
    feed.setQuery(query);
#else
    feed.addQueryItem("fields", YOUTUBE_PLAYLIST_FIELDS);
#endif
    this->setLoading(true);
    QNetworkReply *reply = YouTube::instance()->createReply(feed, this->offset());
    reply->setParent(this);
    this->connect(reply, SIGNAL(finished()), this, SLOT(addPlaylists()));
}

void YouTubePlaylistListModel::search(const QString &query, int order) {
    if (!query.isEmpty()) {
        this->setQuery(query);
        this->setOrder(order);
    }

    this->setLoading(true);
    QNetworkReply *reply = YouTube::instance()->createSearchReply(Queries::Playlists, this->query(), this->offset(), this->order());
    reply->setParent(this);
    this->connect(reply, SIGNAL(finished()), this, SLOT(addPlaylists()));
}

void YouTubePlaylistListModel::addPlaylists() {
    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());

    if (!reply) {
        this->setLoading(false);
        this->setError(true);
        return;
    }

    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();

    if (statusCode == 401) {
        this->connect(YouTube::instance(), SIGNAL(accessTokenRefreshed(QString)), this, SLOT(getPlaylists()));
        this->connect(YouTube::instance(), SIGNAL(refreshError()), this, SLOT(onError()));
        YouTube::instance()->refreshAccessToken();
    }
    else {
        QDomDocument doc;
        doc.setContent(reply->readAll());
        QDomNodeList entries = doc.elementsByTagName("entry");

        for (int i = 0; i < entries.size(); i++) {
            PlaylistItem *playlist = new PlaylistItem;
            playlist->loadYouTubePlaylist(entries.at(i));
            QMetaObject::invokeMethod(this, "appendPlaylist", Q_ARG(QSharedPointer<PlaylistItem>, QSharedPointer<PlaylistItem>(playlist)));
        }

        QTimer::singleShot(1000, this, SLOT(stopLoading()));
        this->setMoreResults((!doc.namedItem("feed").namedItem("link").isNull()) && (!entries.isEmpty()));
        this->setOffset(this->offset() + 30);

        this->disconnect(YouTube::instance(), SIGNAL(accessTokenRefreshed(QString)), this, SLOT(getPlaylists()));
        this->disconnect(YouTube::instance(), SIGNAL(refreshError()), this, SLOT(onError()));
    }

    emit countChanged(this->rowCount());

    reply->deleteLater();
}

void YouTubePlaylistListModel::getMorePlaylists() {
    if ((this->moreResults()) && (!this->loading())) {
        if (!this->query().isEmpty()) {
            this->search();
        }
        else {
            this->getPlaylists();
        }
    }
}
