#include "avatarlabel.h"
#include <QPainter>
#include <QImage>

AvatarLabel::AvatarLabel(QWidget *parent) :
    QWidget(parent),
    m_interactive(false),
    m_pressed(false),
    m_allowClick(true)
{
    this->setFixedSize(144, 178);
}

void AvatarLabel::setName(const QString &name) {
    m_name = name;
    this->repaint();
}

void AvatarLabel::setImage(const QImage &image) {
    m_image = image;
    this->repaint();
}

void AvatarLabel::mousePressEvent(QMouseEvent *event) {
    if (this->interactive()) {
        event->accept();
        m_pressed = true;
        this->repaint();
    }
    else {
        event->ignore();
    }
}

void AvatarLabel::mouseReleaseEvent(QMouseEvent *event) {
    if ((this->interactive()) && (rect().contains(event->pos()))) {
        event->accept();
        m_pressed = false;
        this->repaint();

        if (m_allowClick) {
            emit clicked();
            m_allowClick = false;
            this->startTimer(500);
        }
    }
    else {
        event->ignore();
    }
}

void AvatarLabel::leaveEvent(QEvent *event) {
    event->accept();
    m_pressed = false;
    this->repaint();
}

void AvatarLabel::paintEvent(QPaintEvent *event) {
    Q_UNUSED(event)

    QPainter painter(this);

    if (!this->image().isNull()) {
        QRect imageRect = rect();
        imageRect.setTopLeft(QPoint(imageRect.left() + 8, imageRect.top() + 8));
        imageRect.setSize(QSize(128, 128));

        painter.drawImage(imageRect, this->image());
    }

    if (m_pressed) {
        painter.drawImage(this->rect(), QImage("/etc/hildon/theme/images/ContactsAppletFrameActive.png"));
    }
    else {
        painter.drawImage(this->rect(), QImage("/etc/hildon/theme/images/ContactsAppletFrame.png"));
    }

    QRect textRect = this->rect();
    textRect.setTopLeft(QPoint(textRect.left() + 8, textRect.top() + 137));
    textRect.setSize(QSize(128, 36));

    QFont font;
    font.setPixelSize(18);
    painter.setFont(font);
    painter.setClipRect(textRect);

    if (!this->name().isEmpty()) {
        painter.drawText(textRect, Qt::AlignCenter, this->name());
    }
}

void AvatarLabel::timerEvent(QTimerEvent *event) {
    m_allowClick = true;
    event->accept();
    this->killTimer(event->timerId());
}
