#include "dailymotionplaylistdelegate.h"
#include "../base/abstractplaylistlistmodel.h"
#include <QApplication>
#include <QStaticText>
#include <QPainter>
#include <QMaemo5Style>

DailymotionPlaylistDelegate::DailymotionPlaylistDelegate(QObject *parent) :
    QStyledItemDelegate(parent)
{
}

void DailymotionPlaylistDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    painter->save();

    if ((option.state) & (QStyle::State_Selected)) {
        painter->drawImage(option.rect, QImage("/etc/hildon/theme/images/TouchListBackgroundPressed.png"));
    }
    else {
        painter->drawImage(option.rect, QImage("/etc/hildon/theme/images/TouchListBackgroundNormal.png"));
    }

    QRect titleRect = option.rect;
    titleRect.setLeft(titleRect.left() + 5);
    titleRect.setRight(titleRect.right() - 5);
    titleRect.setTop(titleRect.top() + 5);
    titleRect.setHeight(60);

    painter->drawText(titleRect, Qt::TextWordWrap, index.data(AbstractPlaylistListModel::TitleRole).toString());

    QRect infoRect = option.rect;
    infoRect.setLeft(titleRect.left());
    infoRect.setRight(titleRect.right());
    infoRect.setTop(infoRect.bottom() - 75);
    infoRect.setBottom(infoRect.bottom() - 5);

    QFont smallFont;
    smallFont.setPixelSize(18);
    painter->setFont(smallFont);
    painter->setPen(QMaemo5Style::standardColor("SecondaryTextColor"));

    QString description = index.data(AbstractPlaylistListModel::DescriptionRole).toString();

    QStaticText descriptionText(description.isEmpty() ? tr("No description") : description);
    descriptionText.setTextWidth(infoRect.width());

    int descriptionHeight = descriptionText.size().height();

    if (descriptionHeight <= infoRect.height()) {
        descriptionText.setTextOption(QTextOption(Qt::AlignBottom));
        infoRect.setTop(infoRect.bottom() - descriptionHeight);
    }

    painter->setClipRect(infoRect);
    painter->drawStaticText(infoRect.topLeft(), descriptionText);
    painter->restore();
}

QSize DailymotionPlaylistDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const {
    Q_UNUSED(index)

    return QSize(option.rect.width(), 150);
}
