#include "facebookauthdialog.h"
#include "../base/sharing.h"
#include "webview.h"
#include <QProgressBar>
#include <QGridLayout>

FacebookAuthDialog::FacebookAuthDialog(QWidget *parent) :
    AbstractDialog(parent),
    m_webView(new WebView(this)),
    m_progressBar(new QProgressBar(this)),
    m_grid(new QGridLayout(this))
{
    this->setWindowTitle(tr("facebook authorisation"));

    m_grid->addWidget(m_webView, 0, 0);
    m_grid->addWidget(m_progressBar, 1, 0);

    m_webView->setUrl(Sharing::instance()->facebookAuthUrl());
    m_webView->enableSelectionSuppressor();
    m_progressBar->setTextVisible(false);

    this->connect(m_webView, SIGNAL(urlChanged(QUrl)), this, SLOT(checkUrlForToken(QUrl)));
    this->connect(m_webView, SIGNAL(loadProgress(int)), m_progressBar, SLOT(setValue(int)));
    this->connect(m_webView, SIGNAL(loadStarted()), m_progressBar, SLOT(show()));
    this->connect(m_webView, SIGNAL(loadFinished(bool)), m_progressBar, SLOT(hide()));
    this->connect(this, SIGNAL(rejected()), Sharing::instance(), SLOT(cancelFacebookSignIn()));

    this->onOrientationChanged();
}

void FacebookAuthDialog::checkUrlForToken(const QUrl &url) {
    QString urlString = url.toString();

    if ((urlString.startsWith("http://www.facebook.com/connect/login_success.html")) || (urlString.startsWith("https://www.facebook.com/connect/login_success.html"))) {
        Sharing::instance()->signInToFacebook(url);
        this->accept();
    }
}
