#include "homewindow.h"
#include "dbusadaptor.h"
#include "../base/transferitem.h"
#include "resourceloader.h"
#include "../base/definitions.h"
#include "../base/session.h"
#include "../base/settings.h"
#include "../base/commentitem.h"
#include "../base/groupitem.h"
#include "../base/playlistitem.h"
#include "../base/useritem.h"
#include <QApplication>
#include <QTranslator>
#include <QSsl>
#include <QSslConfiguration>

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    app.setApplicationName("cuteTube");
    app.setApplicationVersion(VERSION_NUMBER);

    qRegisterMetaType< QSharedPointer<CommentItem> >("QSharedPointer<CommentItem>");
    qRegisterMetaType< QSharedPointer<GroupItem> >("QSharedPointer<GroupItem>");
    qRegisterMetaType< QSharedPointer<PlaylistItem> >("QSharedPointer<PlaylistItem>");
    qRegisterMetaType< QSharedPointer<TransferItem> >("QSharedPointer<TransferItem>");
    qRegisterMetaType< QSharedPointer<UserItem> >("QSharedPointer<UserItem>");
    qRegisterMetaType< QSharedPointer<VideoItem> >("QSharedPointer<VideoItem>");

    QSslConfiguration config = QSslConfiguration::defaultConfiguration();
    config.setProtocol(QSsl::TlsV1);
    QSslConfiguration::setDefaultConfiguration(config);

    new Session;

    ResourceLoader loader;
    new DBusAdaptor(&loader);
    QDBusConnection connection = QDBusConnection::sessionBus();
    connection.registerService("com.maemo.cuteTube");
    connection.registerObject("/", &loader);

    if (!Settings::instance()->language().isEmpty()) {
        QTranslator translator;
        translator.load(QString("/opt/cutetube/translations/cutetube_%1").arg(Settings::instance()->language()));
        app.installTranslator(&translator);
    }

    HomeWindow window;
    window.show();

    QObject::connect(&app, SIGNAL(aboutToQuit()), Session::instance(), SLOT(deleteLater()));

    return app.exec();
}
