#include "vimeogrouplistdelegate.h"
#include "../base/abstractgrouplistmodel.h"
#include <QApplication>
#include <QStaticText>
#include <QPainter>
#include <QMaemo5Style>

VimeoGroupListDelegate::VimeoGroupListDelegate(QObject *parent) :
    QStyledItemDelegate(parent)
{
}

void VimeoGroupListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    painter->save();

    if ((option.state) & (QStyle::State_Selected)) {
        painter->drawImage(option.rect, QImage("/etc/hildon/theme/images/TouchListBackgroundPressed.png"));
    }
    else {
        painter->drawImage(option.rect, QImage("/etc/hildon/theme/images/TouchListBackgroundNormal.png"));
    }

    QRect titleRect = option.rect;
    titleRect.setLeft(titleRect.left() + 130);
    titleRect.setRight(titleRect.right() - 5);
    titleRect.setTop(titleRect.top() + 5);
    titleRect.setHeight(60);

    painter->drawText(titleRect, Qt::TextWordWrap, index.data(AbstractGroupListModel::TitleRole).toString());

    QFont smallFont;
    smallFont.setPixelSize(18);
    painter->setFont(smallFont);

    titleRect.moveTop(titleRect.bottom());

    painter->drawText(titleRect, QString("%1 videos").arg(index.data(AbstractGroupListModel::VideoCountRole).toInt()));

    QRect infoRect = option.rect;
    infoRect.setLeft(titleRect.left());
    infoRect.setRight(titleRect.right());
    infoRect.setTop(infoRect.bottom() - 75);
    infoRect.setBottom(infoRect.bottom() - 5);

    painter->setPen(QMaemo5Style::standardColor("SecondaryTextColor"));

    QString description = index.data(AbstractGroupListModel::DescriptionRole).toString();

    QStaticText descriptionText(description.isEmpty() ? tr("No description") : description);
    descriptionText.setTextWidth(infoRect.width());

    int descriptionHeight = descriptionText.size().height();

    if (descriptionHeight <= infoRect.height()) {
        descriptionText.setTextOption(QTextOption(Qt::AlignBottom));
        infoRect.setTop(infoRect.bottom() - descriptionHeight);
    }

    painter->setClipRect(infoRect);
    painter->drawStaticText(infoRect.topLeft(), descriptionText);

    QRect imageRect = option.rect;
    imageRect.setLeft(imageRect.left() + 5);
    imageRect.setTop(imageRect.top() + 5);
    imageRect.setWidth(120);
    imageRect.setHeight(90);

    painter->setClipping(false);

    QImage image = index.data(AbstractGroupListModel::ThumbnailRole).value<QImage>();

    if (!image.isNull()) {
        painter->drawImage(imageRect, image);
    }
    else {
        painter->drawImage(imageRect.left() + 36, imageRect.top() + 21, QImage("/usr/share/icons/hicolor/48x48/hildon/general_video_file.png"));
    }

    painter->drawRect(imageRect);
    painter->restore();
}

QSize VimeoGroupListDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const {
    Q_UNUSED(index)

    return QSize(option.rect.width(), 180);
}
