#include "youtubeplaylistvideoswindow.h"
#include "videolistdelegate.h"
#include "youtubevideoinfowindow.h"
#include "youtubechannelwindow.h"
#include "youtubevideoselectionwindow.h"
#include "youtubeplaylistdialog.h"
#include "videoplaybackwindow.h"
#include "resourceloader.h"
#include "../base/youtubevideolistmodel.h"
#include "listview.h"
#include "thumbnaillabel.h"
#include "separatorlabel.h"
#include "../base/youtube.h"
#include "../base/transfermanager.h"
#include "../base/playbacklistmodel.h"
#include "../base/settings.h"
#include "../base/urlgrabber.h"
#include "../base/thumbnailcache.h"
#include "../base/utils.h"
#include <QAction>
#include <QLabel>
#include <QScrollArea>
#include <QPushButton>
#include <QMenuBar>
#include <QGridLayout>
#include <QScrollBar>
#include <QTimer>
#include <QDesktopServices>

YouTubePlaylistVideosWindow::YouTubePlaylistVideosWindow(QSharedPointer<PlaylistItem> playlist, QWidget *parent) :
    StackedWindow(parent),
    m_playlist(playlist),
    m_listView(new ListView(this)),
    m_model(new YouTubeVideoListModel(this)),
    m_deleteFromPlaylistAction(this->menuBar()->addAction(tr("Delete from playlist"), this, SLOT(deleteVideosFromPlaylist()))),
    m_addToFavouritesAction(this->menuBar()->addAction(tr("Add to favourites"), this, SLOT(addVideosToFavourites()))),
    m_addToPlaylistAction(this->menuBar()->addAction(tr("Add to playlist"), this, SLOT(addVideosToPlaylist()))),
    m_watchLaterAction(this->menuBar()->addAction(tr("Watch later"), this, SLOT(addVideosToWatchLaterPlaylist()))),
    m_addToQueueAction(this->menuBar()->addAction(tr("Add to playback queue"), this, SLOT(queueVideos()))),
    m_videoDownloadAction(this->menuBar()->addAction(tr("Download"), this, SLOT(addVideoDownloads()))),
    m_audioDownloadAction(this->menuBar()->addAction(tr("Download (audio)"), this, SLOT(addAudioDownloads()))),
    m_contextMenu(new QMenu(this)),
    m_deleteFromPlaylistContextAction(m_contextMenu->addAction(tr("Delete from playlist"), this, SLOT(deleteVideoFromPlaylist()))),
    m_addToFavouritesContextAction(m_contextMenu->addAction(tr("Add to favourites"), this, SLOT(addVideoToFavourites()))),
    m_addToPlaylistContextAction(m_contextMenu->addAction(tr("Add to playlist"), this, SLOT(showPlaylistDialog()))),
    m_watchLaterContextAction(m_contextMenu->addAction(tr("Watch later"), this, SLOT(addVideoToWatchLaterPlaylist()))),
    m_addToQueueContextAction(m_contextMenu->addAction(tr("Add to playback queue"), this, SLOT(queueVideo()))),
    m_videoDownloadContextAction(m_contextMenu->addAction(tr("Download"), this, SLOT(addVideoDownload()))),
    m_audioDownloadContextAction(m_contextMenu->addAction(tr("Download (audio)"), this, SLOT(addAudioDownload()))),
    m_titleLabel(new QLabel(QString("<p style='font-size:32px;'>%1</p>").arg(m_playlist.data()->title()), this)),
    m_descriptionLabel(new QLabel(m_playlist.data()->description().isEmpty() ? tr("No description") : Utils::toRichText(m_playlist.data()->description()), this)),
    m_dateLabel(new QLabel(QString("%1 - %2").arg(m_playlist.data()->date()).arg(m_playlist.data()->artist()), this)),
    m_thumbnail(new ThumbnailLabel(this)),
    m_avatar(new ThumbnailLabel(this)),
    m_playButton(new QPushButton(tr("Play all"), this)),
    m_separator1(new SeparatorLabel(tr("Description"), this)),
    m_separator2(new SeparatorLabel(QString("-- %1").arg(tr("videos")), this)),
    m_scrollArea(new QScrollArea(this))
{
    this->setWindowTitle(tr("Playlist"));
    this->setCentralWidget(m_scrollArea);

    VideoListDelegate *delegate = new VideoListDelegate(m_listView);

    m_listView->setModel(m_model);
    m_listView->setItemDelegate(delegate);
    m_descriptionLabel->setWordWrap(true);
    m_dateLabel->setWordWrap(true);

    QWidget *scrollWidget = new QWidget(m_scrollArea);
    QGridLayout *grid = new QGridLayout(scrollWidget);

    m_thumbnail->setInteractive(false);
    m_thumbnail->setFixedSize(160, 120);
    m_avatar->setFixedSize(60, 60);
    m_playButton->setFixedWidth(150);

    grid->setContentsMargins(0, 0, 0, 0);
    grid->addItem(new QSpacerItem(10, 10, QSizePolicy::Maximum), 0, 0);
    grid->addItem(new QSpacerItem(10, 10, QSizePolicy::Maximum), 1, 0);
    grid->addItem(new QSpacerItem(10, 10, QSizePolicy::Maximum), 3, 0);
    grid->addWidget(m_titleLabel, 0, 1, 1, 3, Qt::AlignTop);
    grid->addWidget(m_thumbnail, 1, 1, Qt::AlignTop);
    grid->addWidget(m_avatar, 1, 2, Qt::AlignTop);
    grid->addWidget(m_dateLabel, 1, 3, Qt::AlignTop);
    grid->addWidget(m_playButton, 2, 3, Qt::AlignRight);
    grid->addWidget(m_separator1, 3, 0, 1, 4);
    grid->addWidget(m_descriptionLabel, 4, 1, 1, 3);
    grid->addWidget(m_separator2, 5, 0, 1, 4);
    grid->addWidget(m_listView, 6, 0, 1, 4);

    m_scrollArea->setWidgetResizable(true);
    m_scrollArea->setWidget(scrollWidget);
    m_scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    this->connect(m_model, SIGNAL(loadingChanged(bool)), this, SLOT(onLoadingChanged(bool)));
    this->connect(m_model, SIGNAL(countChanged(int)), this, SLOT(onCountChanged(int)));
    this->connect(delegate, SIGNAL(clicked(QModelIndex)), this, SLOT(onItemClicked(QModelIndex)));
    this->connect(delegate, SIGNAL(play(QModelIndex)), this, SLOT(playVideo(QModelIndex)));
    this->connect(m_listView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
    this->connect(m_avatar, SIGNAL(clicked()), this, SLOT(showUserChannel()));
    this->connect(m_playButton, SIGNAL(clicked()), this, SLOT(playPlaylist()));
    this->connect(m_descriptionLabel, SIGNAL(linkActivated(QString)), this, SLOT(parseActivatedLink(QString)));

    m_model->getPlaylistVideos(m_playlist.data()->playlistId());

    this->setThumbnails();
    this->setMenuActions();
    this->onOrientationChanged();
}

void YouTubePlaylistVideosWindow::showEvent(QShowEvent *event) {
    StackedWindow::showEvent(event);

    if ((m_model) && (m_model->refreshRequired())) {
        QTimer::singleShot(1000, m_model, SLOT(reset()));
    }
}

void YouTubePlaylistVideosWindow::setLandscapeLayout() {
    m_scrollArea->widget()->setMaximumWidth(800);
}

void YouTubePlaylistVideosWindow::setPortraitLayout() {
    m_scrollArea->widget()->setMaximumWidth(480);
}

void YouTubePlaylistVideosWindow::setThumbnails() {
    QImage thumbnail = ThumbnailCache::instance()->thumbnail(m_playlist.data()->thumbnailUrl(), QSize(160, 120));
    QImage avatar = ThumbnailCache::instance()->thumbnail(m_playlist.data()->artistAvatarUrl(), QSize(60, 60));

    if ((thumbnail.isNull()) || (avatar.isNull())) {
        this->connect(ThumbnailCache::instance(), SIGNAL(thumbnailReady()), this, SLOT(setThumbnails()));
    }
    else {
        this->disconnect(ThumbnailCache::instance(), SIGNAL(thumbnailReady()), this, SLOT(setThumbnails()));
    }
    if (!thumbnail.isNull()) {
        m_thumbnail->setImage(thumbnail);
    }
    if (!avatar.isNull()) {
        m_avatar->setImage(avatar);
    }
}

void YouTubePlaylistVideosWindow::setMenuActions() {
    bool signedIn = YouTube::instance()->userSignedIn();

    m_addToFavouritesAction->setEnabled(signedIn);
    m_addToPlaylistAction->setEnabled(signedIn);
    m_watchLaterAction->setEnabled(signedIn);
}

void YouTubePlaylistVideosWindow::onLoadingChanged(bool loading) {
    this->toggleBusy(loading);
    m_playButton->setEnabled((!loading) && (m_model->rowCount() > 0) && (Settings::instance()->mediaPlayer() == "cutetube"));

    if (!loading) {
        m_separator2->setText(QString("%1 %2").arg(m_model->rowCount()).arg(tr("videos")));

        if (m_model->rowCount() > 0) {
            m_listView->setFixedHeight(m_model->rowCount() * 150);
        }
    }
}

void YouTubePlaylistVideosWindow::onCountChanged(int count) {
    if (!m_model->loading()) {
        m_separator2->setText(QString("%1 %2").arg(count).arg(tr("videos")));
    }
}

void YouTubePlaylistVideosWindow::deleteVideosFromPlaylist() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Delete from playlist"), SLOT(deleteVideosFromPlaylist()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::addVideosToFavourites() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Add to favourites"), SLOT(addVideosToFavourites()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::addVideosToPlaylist() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Add to playlist"), SLOT(showPlaylistDialog()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::addVideosToWatchLaterPlaylist() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Watch later"), SLOT(addVideosToWatchLaterPlaylist()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::addVideoDownloads() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Download"), SLOT(addVideoDownloads()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::addAudioDownloads() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Download"), SLOT(addAudioDownloads()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::queueVideos() {
    YouTubeVideoSelectionWindow *window = new YouTubeVideoSelectionWindow(m_model, tr("Add to queue"), SLOT(queueVideos()), this);
    window->positionAtIndex(m_listView->indexAt(QPoint(0, m_scrollArea->verticalScrollBar()->value())));
    window->show();
}

void YouTubePlaylistVideosWindow::showContextMenu(const QPoint &pos) {
    m_contextMenu->popup(pos, m_deleteFromPlaylistContextAction);
}

void YouTubePlaylistVideosWindow::deleteVideoFromPlaylist() {
    YouTube::instance()->deleteFromPlaylist(QStringList(m_model->data(m_listView->currentIndex(), AbstractVideoListModel::PlaylistVideoIdRole).toString()), m_model->playlistId());
}

void YouTubePlaylistVideosWindow::addVideoToFavourites() {
    YouTube::instance()->addToFavourites(QStringList(m_model->data(m_listView->currentIndex(), AbstractVideoListModel::VideoIdRole).toString()));
}

void YouTubePlaylistVideosWindow::showPlaylistDialog() {
    YouTubePlaylistDialog *dialog = new YouTubePlaylistDialog(this);
    dialog->open();

    this->connect(dialog, SIGNAL(playlistChosen(QString)), this, SLOT(addVideoToPlaylist(QString)));
    this->connect(dialog, SIGNAL(addToNewPlaylist(QVariantMap)), this, SLOT(addVideoToNewPlaylist(QVariantMap)));
}

void YouTubePlaylistVideosWindow::addVideoToPlaylist(const QString &id) {
    YouTube::instance()->addToPlaylist(QStringList(m_model->data(m_listView->currentIndex(), AbstractVideoListModel::VideoIdRole).toString()), id);
}

void YouTubePlaylistVideosWindow::addVideoToNewPlaylist(const QVariantMap &playlist) {
    YouTube::instance()->createPlaylist(playlist, QStringList(m_model->data(m_listView->currentIndex(), AbstractVideoListModel::VideoIdRole).toString()));
}

void YouTubePlaylistVideosWindow::addVideoToWatchLaterPlaylist() {
    YouTube::instance()->addToWatchLaterPlaylist(QStringList(m_model->data(m_listView->currentIndex(), AbstractVideoListModel::VideoIdRole).toString()));
}

void YouTubePlaylistVideosWindow::addVideoDownload() {
    TransferManager::instance()->addDownloadTransfer(m_model->get(m_listView->currentIndex().row()));
}

void YouTubePlaylistVideosWindow::addAudioDownload() {
    TransferManager::instance()->addDownloadTransfer(m_model->get(m_listView->currentIndex().row()), true);
}

void YouTubePlaylistVideosWindow::queueVideo() {
    PlaybackListModel::playbackQueue()->addVideo(m_model->get(m_listView->currentIndex().row()));
}

void YouTubePlaylistVideosWindow::onItemClicked(const QModelIndex &index) {
    if (index.isValid()) {
        this->showVideoInfo(m_model->get(index.row()));
    }
}

void YouTubePlaylistVideosWindow::showVideoInfo(QSharedPointer<VideoItem> video) {
    YouTubeVideoInfoWindow *window = new YouTubeVideoInfoWindow(video, this);
    window->show();
}

void YouTubePlaylistVideosWindow::playVideo(const QModelIndex &index) {
    if (Settings::instance()->mediaPlayer() == "cutetube") {
        VideoPlaybackWindow *window = new VideoPlaybackWindow(QList< QSharedPointer<VideoItem> >() << m_model->get(index.row()), this);
        window->show();
    }
    else {
        UrlGrabber::instance()->getYouTubeVideoUrl(index.data(AbstractVideoListModel::VideoIdRole).toString());
    }
}

void YouTubePlaylistVideosWindow::playPlaylist() {
    QList< QSharedPointer<VideoItem> > videos;

    for (int i = 0; i < m_model->rowCount(); i++) {
        videos.append(m_model->get(i));
    }

    VideoPlaybackWindow *window = new VideoPlaybackWindow(videos, this);
    window->show();
}

void YouTubePlaylistVideosWindow::showUserChannel() {
    YouTubeChannelWindow *window = new YouTubeChannelWindow(m_playlist.data()->artistId(), this);
    window->show();
}

void YouTubePlaylistVideosWindow::parseActivatedLink(const QString &link) {
    if (!ResourceLoader::instance()->display(link, false)) {
        QDesktopServices::openUrl(link);
    }
}
